__author__ = 'wolkens'

import math
import const, triangle

class StellatedDodecahedron():
    def  __init__(self, ch, pos = 0.0, size = 100.0):
        self.offsetx, self.offsety, self.offsetz = pos, 0.0, const.OFFSETZ # Linear shift in space
        self.channel = ch # L or R
        self.size = size # Size of the fixed vertices
        self.siz2 = size / 2.1 # Size of the flexible vertices
        self.level = 0 # No input
        self.overload = False
        self.delaycounter = 0 # Countdown for overload state
        self.Vertices = {} # Empty dictionary of vertices
        self.Triangles = [] # Empty list of triangles
        self.Peaks = {} # Empty dictionary of peaks
        self.TriangleList = [] # Empty list of triangle indexes (numeric)
        self.rotMatrix = [[0.0 for col in range(3)] for row in range(3)] # Rotation matrix

        phi = 1.618 # Golden ratio
        phi2 = phi * phi # Square

        # Create dodecahedron vertices
        self.Vertices["A"]  = triangle.Vertex(self.size, [   phi,      0,  1/phi], pos, True) # Fixed vertices
        self.Vertices["B"]  = triangle.Vertex(self.size, [  -phi,      0,  1/phi], pos, True)
        self.Vertices["C"]  = triangle.Vertex(self.size, [  -phi,      0, -1/phi], pos, True)
        self.Vertices["D"]  = triangle.Vertex(self.size, [   phi,      0, -1/phi], pos, True)
        self.Vertices["E"]  = triangle.Vertex(self.size, [ 1/phi,    phi,      0], pos, True)
        self.Vertices["F"]  = triangle.Vertex(self.size, [ 1/phi,   -phi,      0], pos, True)
        self.Vertices["G"]  = triangle.Vertex(self.size, [-1/phi,   -phi,      0], pos, True)
        self.Vertices["H"]  = triangle.Vertex(self.size, [-1/phi,    phi,      0], pos, True)
        self.Vertices["I"]  = triangle.Vertex(self.size, [     0,  1/phi,    phi], pos, True)
        self.Vertices["J"]  = triangle.Vertex(self.size, [     0,  1/phi,   -phi], pos, True)
        self.Vertices["K"]  = triangle.Vertex(self.size, [     0, -1/phi,   -phi], pos, True)
        self.Vertices["L"]  = triangle.Vertex(self.size, [     0, -1/phi,    phi], pos, True)
        self.Vertices["M"]  = triangle.Vertex(self.size, [     1,      1,      1], pos, True)
        self.Vertices["N"]  = triangle.Vertex(self.size, [     1,     -1,      1], pos, True)
        self.Vertices["O"]  = triangle.Vertex(self.size, [    -1,     -1,      1], pos, True)
        self.Vertices["P"]  = triangle.Vertex(self.size, [    -1,      1,      1], pos, True)
        self.Vertices["Q"]  = triangle.Vertex(self.size, [    -1,      1,     -1], pos, True)
        self.Vertices["R"]  = triangle.Vertex(self.size, [     1,      1,     -1], pos, True)
        self.Vertices["S"]  = triangle.Vertex(self.size, [     1,     -1,     -1], pos, True)
        self.Vertices["T"]  = triangle.Vertex(self.size, [    -1,     -1,     -1], pos, True)
        self.Vertices["A1"] = triangle.Vertex(self.siz2, [     0,   phi2,    phi], pos, False) # Moving vertices
        self.Vertices["B1"] = triangle.Vertex(self.siz2, [     0,   phi2,   -phi], pos, False)
        self.Vertices["C1"] = triangle.Vertex(self.siz2, [     0,  -phi2,   -phi], pos, False)
        self.Vertices["D1"] = triangle.Vertex(self.siz2, [     0,  -phi2,    phi], pos, False)
        self.Vertices["E1"] = triangle.Vertex(self.siz2, [  phi2,    phi,      0], pos, False)
        self.Vertices["F1"] = triangle.Vertex(self.siz2, [  phi2,   -phi,      0], pos, False)
        self.Vertices["G1"] = triangle.Vertex(self.siz2, [ -phi2,   -phi,      0], pos, False)
        self.Vertices["H1"] = triangle.Vertex(self.siz2, [ -phi2,    phi,      0], pos, False)
        self.Vertices["I1"] = triangle.Vertex(self.siz2, [   phi,      0,   phi2], pos, False)
        self.Vertices["J1"] = triangle.Vertex(self.siz2, [  -phi,      0,   phi2], pos, False)
        self.Vertices["K1"] = triangle.Vertex(self.siz2, [  -phi,      0,  -phi2], pos, False)
        self.Vertices["L1"] = triangle.Vertex(self.siz2, [   phi,      0,  -phi2], pos, False)

        # Create 12 peaks of 5 same color triangles each - total 60 triangles
        self.Peaks["L1"] = ("DSKJR", 255,   0,   0,  0) #    63 Hz
        self.Peaks["B1"] = ("RJQHE", 255,  85,   0,  7) #   100 Hz
        self.Peaks["K1"] = ("TCQJK", 255, 170,   0,  1) #   160 Hz
        self.Peaks["C1"] = ("TKSFG", 255, 255,   0,  8) #   250 Hz
        self.Peaks["F1"] = ("NFSDA", 192, 255,   0,  2) #   400 Hz
        self.Peaks["E1"] = ("DREMA", 128, 255,   0,  9) #   630 Hz
        self.Peaks["A1"] = ("EHPIM",  64, 255,   0,  3) #  1000 Hz
        self.Peaks["G1"] = ("CTGOB",   0, 192,  64, 10) #  4000 Hz
        self.Peaks["H1"] = ("HQCBP",   0, 255,   0,  4) #  2500 Hz
        self.Peaks["D1"] = ("NLOGF",   0, 128, 128,  5) #  6250 Hz
        self.Peaks["I1"] = ("MILNA",   0,  64, 192, 11) # 10000 Hz
        self.Peaks["J1"] = ("PBOLI",   0,   0, 255,  6) # 16000 Hz

        for av in self.Peaks.keys():
            currpenta, currred, currgreen, currblue = self.Peaks[av][0:4]
            currpenta = currpenta + currpenta[0] # Make a closed loop
            p0 = av # Top of the peak
            for i in range(0, 5): # Create 5 triangles
                p1 = currpenta[i] # Get other 2 vertices
                p2 = currpenta[i+1]
                self.Triangles.append(triangle.Triangle(
                    p0, p1, p2, currred, currgreen, currblue, self.Vertices )) # Create triangle
        return # For debug only

    def update(self, dest, levels, angle, axis, dx, dy): # Recalculate and repaint the dodecahedron
        if (max(levels)> 176) and (not self.overload): # Overload starts
            self.overload = True # Start blowup period
            self.delaycounter = const.BLOWUPTIME + const.RELASETIME # Set delay counter
        if self.overload: # Delay
            self.delaycounter = self.delaycounter - 1
            if self.delaycounter == 0: self.overload = False # Back to normal

        sinalfa = math.sin(angle) # Calculate rotation matrix for dodecahedron
        cosalfa = math.cos(angle)
        cosalfaM1 = 1-cosalfa
        a = cosalfa + axis.x * axis.x * cosalfaM1
        b = axis.x * axis.y * cosalfaM1 - axis.z * sinalfa
        c = axis.x * axis.z * cosalfaM1 + axis.y * sinalfa
        p = axis.y * axis.x * cosalfaM1 + axis.z * sinalfa
        q = cosalfa + axis.y * axis.y * cosalfaM1
        r = axis.y * axis.z * cosalfaM1 - axis.x * sinalfa
        u = axis.z * axis.x * cosalfaM1 - axis.y * sinalfa
        v = axis.z * axis.y * cosalfaM1 + axis.x * sinalfa
        w = cosalfa + axis.z * axis.z * cosalfaM1
        self.rotMatrix = [ [a, b, c], [p, q, r], [u, v, w] ]
			
        for vx in self.Vertices: # Identify level
            if vx in self.Peaks:
                currlevel = 1.25 * levels[self.Peaks[vx][4]] # Defined by the peak
            else:
                currlevel = 0 # Fixed vertices
            self.Vertices[vx].behavior(int(currlevel), self.overload) # Update vertices
            self.Vertices[vx].rotate(self.rotMatrix) # Rotate and project to screen

        for i in range(0, 60): # Update all triangles based on new vertex coordinates
            self.Triangles[i].update()
        self.TriangleList = [] # Clear list
        for i in range(0, 60): # Create list of triangles to be drawn
            if self.Triangles[i].visible: # Add visible triangles to the list
                if len(self.TriangleList) == 0: # No triangles in list
                    self.TriangleList.append(i) # Add first triangle
                else:
                    posfound = False # Clear found flag
                    z0 = self.Triangles[i].Center.z # Get depth of current triangle
                    for j in range(0, len(self.TriangleList)): # Check already added triangles
                        if z0 < self.Triangles[self.TriangleList[j]].Center.z: # Insert before this triangle
                            self.TriangleList.insert(j, i) # Add triangle to list depth-sorted (near to far order)
                            posfound = True # Set flag
                            break
                    if not posfound: self.TriangleList.append(i) # Add triangle to the end of the list
        for i in range(0, len(self.TriangleList)): # Draw in reverse order - from far to near
            self.Triangles[self.TriangleList[len(self.TriangleList) - i - 1]].draw(dest, dx, dy, self.overload)
